# -*- coding: utf-8 -*-

import re
from lxml import etree
from urllib.parse import urlparse, unquote
from .parser import Parser


class Javday(Parser):
    source = 'javday'

    expr_url = '/html/head/meta[@property="og:url"]/@content'
    expr_cover = '/html/head/meta[@property="og:image"]/@content'
    expr_tags = '/html/head/meta[@name="keywords"]/@content'
    expr_title = "/html/head/title/text()"
    expr_actor = "//span[@class='vod_actor']/a/text()"
    expr_studio = '//span[@class="producer"]/a/text()'
    expr_number = '//span[@class="jpnum"]/text()'

    def extraInit(self):
        self.imagecut = 4
        self.uncensored = True

    def search(self, number):
        self.number = number.strip().upper()
        if self.specifiedUrl:
            self.detailurl = self.specifiedUrl
        else:
            self.detailurl = "https://javday.tv/videos/" + self.number.replace("-","") + "/"
        self.htmlcode = self.getHtml(self.detailurl)
        if self.htmlcode == 404:
            return 404
        htmltree = etree.fromstring(self.htmlcode, etree.HTMLParser())
        self.detailurl = self.getTreeElement(htmltree, self.expr_url)

        result = self.dictformat(htmltree)
        return result

    def getTitle(self, htmltree):
        title = super().getTitle(htmltree)
        # 删除番号和网站名
        result = title.replace(self.number,"").replace("- JAVDAY.TV","").strip()
        return result



# -*- coding: utf-8 -*-


from urllib.parse import quote, urljoin
from .parser import Parser


class Tmdb(Parser):
    """
    两种实现,带apikey与不带key
    apikey
    """
    source = 'tmdb'

    expr_title = '//head/meta[@property="og:title"]/@content'
    expr_release = '//div/span[@class="release"]/text()'
    expr_cover = '//head/meta[@property="og:image"]/@content'
    expr_outline = '//head/meta[@property="og:description"]/@content'

    def extraInit(self):
        imagecut = 0
        apikey = None

    def queryNumberUrl(self, number: str):
        """ 区分 ID 与 名称
        """
        if number.isdigit():
            id  = number
            detailurl = "https://www.themoviedb.org/movie/" + id + "?language=zh-CN"
        else:
            queryUrl = "https://www.themoviedb.org/search?query=" + quote(number, encoding="utf8") + "&language=zh-CN"
            queryTree = self.getHtmlTree(queryUrl)
            cardsexpr = '//div[@class="card v4 tight"]/div/div[2]/div/div/div/a'
            cardselement = self.getTreeAll(queryTree, cardsexpr)
            # 获取所有结果列表，最小到a标签，h2为标题，href为url
            # cardselement[0].xpath('@href') / cardselement[0].xpath('h2/text()')
            if len(cardselement):
                firstUrl = self.getTreeElement(cardselement[0], '@href')
                detailurl = urljoin("https://www.themoviedb.org", firstUrl)
            else:
                raise Exception("TMDB: Can't find detail page URL")
        return detailurl


    def getCover(self, htmltree):
        return "https://www.themoviedb.org" + self.getTreeElement(htmltree, self.expr_cover)




# -*- coding: utf-8 -*-

import re
import secrets
from urllib.parse import urljoin
from .httprequest import get_html_by_form
from .parser import Parser


class Xcity(Parser):
    source = 'xcity'

    expr_number = '//*[@id="hinban"]/text()'
    expr_title = '//*[@id="program_detail_title"]/text()'
    expr_actor = '//ul/li[@class="credit-links"]/a/text()'
    expr_actor_link = '//ul/li[@class="credit-links"]/a'
    expr_actorphoto = '//div[@class="frame"]/div/p/img/@src'
    expr_studio = '//*[@id="avodDetails"]/div/div[3]/div[2]/div/ul[1]/li[4]/a/span/text()'
    expr_studio2 = '//strong[contains(text(),"片商")]/../following-sibling::span/a/text()'
    expr_runtime = '//span[@class="koumoku" and text()="収録時間"]/../text()'
    expr_label = '//*[@id="avodDetails"]/div/div[3]/div[2]/div/ul[1]/li[5]/a/span/text()'
    expr_release = '//*[@id="avodDetails"]/div/div[3]/div[2]/div/ul[1]/li[2]/text()'
    expr_tags = '//span[@class="koumoku" and text()="ジャンル"]/../a[starts-with(@href,"/avod/genre/")]/text()'
    expr_cover = '//*[@id="avodDetails"]/div/div[3]/div[1]/p/a/@href'
    expr_director = '//*[@id="program_detail_director"]/text()'
    expr_series = "//span[contains(text(),'シリーズ')]/../a/span/text()"
    expr_series2 = "//span[contains(text(),'シリーズ')]/../span/text()"
    expr_extrafanart = '//div[@id="sample_images"]/div/a/@href'
    expr_outline = '//head/meta[@property="og:description"]/@content'

    def queryNumberUrl(self, number):
        xcity_number = number.replace('-','')
        query_result, browser = get_html_by_form(
            'https://xcity.jp/' + secrets.choice(['sitemap/','policy/','law/','help/','main/']),
            fields = {'q' : xcity_number.lower()},
            cookies=self.cookies, proxies=self.proxies, verify=self.verify,
            return_type = 'browser')
        if not query_result or not query_result.ok:
            raise ValueError("xcity.py: page not found")
        prelink = browser.links('avod\/detail')[0]['href']
        return urljoin('https://xcity.jp', prelink)

    def getStudio(self, htmltree):
        return super().getStudio(htmltree).strip('+').replace("', '", '').replace('"', '')

    def getRuntime(self, htmltree):
        return self.getTreeElement(htmltree, self.expr_runtime, 1).strip()

    def getRelease(self, htmltree):
        try:
            result = self.getTreeElement(htmltree, self.expr_release, 1)
            return re.findall('\d{4}/\d{2}/\d{2}', result)[0].replace('/','-')
        except:
            return ''

    def getCover(self, htmltree):
        try:
            result = super().getCover(htmltree)
            return 'https:' + result
        except:
            return ''
    
    def getDirector(self, htmltree):
        try:
            result = super().getDirector(htmltree).replace(u'\n','').replace(u'\t', '')
            return result
        except:
            return ''

    def getActorPhoto(self, htmltree):
        treea = self.getTreeAll(htmltree, self.expr_actor_link)
        t = {i.text.strip(): i.attrib['href'] for i in treea}
        o = {}
        for k, v in t.items():
            actorpageUrl = "https://xcity.jp" + v
            try:
                adtree = self.getHtmlTree(actorpageUrl)
                picUrl = self.getTreeElement(adtree, self.expr_actorphoto)
                if 'noimage.gif' in picUrl:
                    continue
                o[k] = urljoin("https://xcity.jp", picUrl)
            except:
                pass
        return o

    def getExtrafanart(self, htmltree):
        arts = self.getTreeAll(htmltree, self.expr_extrafanart)
        extrafanart = []
        for i in arts:
            i = "https:" + i
            extrafanart.append(i)
        return extrafanart



# -*- coding: utf-8 -*-

# Set default logging handler to avoid "No handler found" warnings.
import logging
from logging import NullHandler
logging.getLogger(__name__).addHandler(NullHandler())

from .api import search, getSupportedSources



# -*- coding: utf-8 -*-


from .parser import Parser


class Imdb(Parser):
    source = 'imdb'

    expr_title = '//h1[@data-testid="hero-title-block__title"]/text()'
    expr_release = '//a[contains(text(),"Release date")]/following-sibling::div[1]/ul/li/a/text()'
    expr_cover = '//head/meta[@property="og:image"]/@content'
    expr_outline = '//head/meta[@property="og:description"]/@content'
    expr_actor = '//h3[contains(text(),"Top cast")]/../../../following-sibling::div[1]/div[2]/div/div/a/text()'
    expr_tags = '//div[@data-testid="genres"]/div[2]/a/ul/li/text()'

    def extraInit(self):
        imagecut = 0

    def queryNumberUrl(self, number):
        """
        TODO 区分 ID 与 名称
        """
        id  = number
        movieUrl = "https://www.imdb.com/title/" + id
        return movieUrl
